<?php

declare(strict_types = 1);

namespace Drupal\gutenberg_group_media\Plugin\GroupFinder;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Path\PathValidatorInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Url;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface;
use Drupal\group_finder\GroupFinderBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Base class for gutenberg media routes group finder.
 */
abstract class GutenbergMediaRoutesBase extends GroupFinderBase {

  /**
   * The group relation type manager.
   *
   * @var \Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface
   */
  protected GroupRelationTypeManagerInterface $groupRelationTypeManager;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * The path validator.
   *
   * @var \Drupal\Core\Path\PathValidatorInterface
   */
  protected PathValidatorInterface $pathValidator;

  /**
   * The group entity.
   *
   * @var \Drupal\group\Entity\GroupInterface
   */
  protected GroupInterface $group;

  /**
   * GutenbergMediaRoutesBase constructor.
   *
   * @param array $configuration
   *   Plugin configuration.
   * @param string $plugin_id
   *   Plugin id.
   * @param mixed $plugin_definition
   *   Plugin definition.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match.
   * @param \Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface $group_relation_type_manager
   *   The group relation type manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack used to retrieve the current request.
   * @param \Drupal\Core\Path\PathValidatorInterface $path_validator
   *   The path validator service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, RouteMatchInterface $route_match, GroupRelationTypeManagerInterface $group_relation_type_manager, EntityTypeManagerInterface $entity_type_manager, RequestStack $request_stack, PathValidatorInterface $path_validator) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $route_match);
    $this->groupRelationTypeManager = $group_relation_type_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->requestStack = $request_stack;
    $this->pathValidator = $path_validator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_route_match'),
      $container->get('group_relation_type.manager'),
      $container->get('entity_type.manager'),
      $container->get('request_stack'),
      $container->get('path.validator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(): bool {
    $gutenberg_media_routes = [
      'gutenberg.media.dialog',
      'gutenberg.media.upload',
    ];
    return in_array($this->routeMatch->getRouteName(), $gutenberg_media_routes);
  }

  /**
   * {@inheritdoc}
   */
  public function getGroup(): ?GroupInterface {
    return $this->group ?? NULL;
  }

  /**
   * Returns a URL object, if the path is valid and accessible.
   *
   * @return \Drupal\Core\Url|bool
   *   The \Drupal\Core\Url object, or FALSE if the path is not valid.
   */
  protected function getUrlObject(): Url|bool {
    $request = $this->requestStack->getCurrentRequest();
    if ($referer = $request->server->get('HTTP_REFERER')) {
      $original_request = Request::create($referer);
      $path = mb_substr($original_request->getRequestUri(), strlen($request->getBasePath()));
      return $this->pathValidator->getUrlIfValid($path);
    }

    return FALSE;
  }

}
