<?php

declare(strict_types = 1);

namespace Drupal\gutenberg_group_media\Service;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\group_finder\GroupFinderProviderInterface;
use Drupal\group_media_library\GroupMediaLibraryState;
use Drupal\gutenberg\DataProvider\EntityDataProviderManager;
use Drupal\gutenberg\GutenbergMediaLibraryUiBuilder;
use Drupal\gutenberg\MediaEntityRendererInterface;
use Drupal\gutenberg\MediaTypeGuesserInterface;
use Drupal\gutenberg\MediaUploaderInterface;
use Drupal\gutenberg\Persistence\MediaTypePersistenceManagerInterface;
use Drupal\gutenberg\Service\MediaService;
use Drupal\gutenberg\Service\MediaTypeNotFoundException;
use Drupal\media_library\MediaLibraryState;

/**
 * Decorator for media service class.
 */
class MediaServiceDecorator extends MediaService {

  /**
   * The original media service.
   *
   * @var \Drupal\gutenberg\Service\MediaService
   */
  protected MediaService $mediaService;

  /**
   * The group finder provider service.
   *
   * @var \Drupal\group_finder\GroupFinderProviderInterface
   */
  protected GroupFinderProviderInterface $groupFinderProvider;

  /**
   * MediaServiceDecorator constructor.
   *
   * @param \Drupal\gutenberg\Service\MediaService $media_service
   *   The original media service.
   * @param \Drupal\gutenberg\MediaTypeGuesserInterface $media_type_guesser
   *   The media type guesser.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\gutenberg\Persistence\MediaTypePersistenceManagerInterface $media_type_persistence_manager
   *   The media type persistence manager.
   * @param \Drupal\gutenberg\DataProvider\EntityDataProviderManager $entity_data_provider_manager
   *   The entity data provider manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\gutenberg\MediaUploaderInterface $media_uploader
   *   The media uploader.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle service.
   * @param \Drupal\gutenberg\MediaEntityRendererInterface $media_entity_renderer
   *   The media entity renderer.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   * @param \Drupal\group_finder\GroupFinderProviderInterface $group_finder_provider
   *   The group finder provider service.
   */
  public function __construct(
    MediaService $media_service,
    MediaTypeGuesserInterface $media_type_guesser,
    EntityTypeManagerInterface $entity_type_manager,
    MediaTypePersistenceManagerInterface $media_type_persistence_manager,
    EntityDataProviderManager $entity_data_provider_manager,
    ModuleHandlerInterface $module_handler,
    MediaUploaderInterface $media_uploader,
    RendererInterface $renderer,
    EntityTypeBundleInfoInterface $entity_type_bundle_info,
    MediaEntityRendererInterface $media_entity_renderer,
    Connection $connection,
    EntityDisplayRepositoryInterface $entity_display_repository,
    GroupFinderProviderInterface $group_finder_provider
  ) {
    parent::__construct($media_type_guesser, $entity_type_manager, $media_type_persistence_manager, $entity_data_provider_manager, $module_handler, $media_uploader, $renderer, $entity_type_bundle_info, $media_entity_renderer, $connection, $entity_display_repository);
    $this->mediaService = $media_service;
    $this->groupFinderProvider = $group_finder_provider;
  }

  /**
   * Renders media library dialog for Gutenberg editor.
   *
   * @param array $media_types
   *   Array of media types.
   * @param array|null $media_bundles
   *   Array of media bundles.
   *
   * @return \Drupal\Component\Render\MarkupInterface
   *   The rendered element.
   *
   * @throws \Drupal\gutenberg\Service\MediaTypeNotFoundException
   */
  public function renderDialog(array $media_types, array $media_bundles = NULL): MarkupInterface {
    $media_types = array_filter($media_types)
      ? $media_types
      : ['application', 'image', 'audio', 'video', 'text'];
    $allowed_media_type_ids = [];
    foreach ($media_types as $media_type) {
      $allowed_media_type_ids = array_merge($allowed_media_type_ids, $this->mediaTypeGuesser->guess($media_type));
    }

    if (!empty($media_bundles)) {
      // Filter by bundle.
      $allowed_media_type_ids = array_intersect($allowed_media_type_ids, $media_bundles);
    }

    if (!$allowed_media_type_ids) {
      throw new MediaTypeNotFoundException();
    }

    $state = MediaLibraryState::create('gutenberg.media_library.opener', array_unique($allowed_media_type_ids), reset($allowed_media_type_ids), 1);

    /** @var \Drupal\group_finder\GroupFinderInterface $group_finder */
    if ($group_finder = $this->groupFinderProvider->get()) {
      $group_state = GroupMediaLibraryState::create(
        $state,
        $group_finder->getPluginId(),
        $group_finder->getGroup()->getGroupType()->id(),
        $group_finder->getGroup()->uuid() ?? '0',
        $group_finder->getGroup()->id() ? $group_finder->Getgroup()->id() : '0',
        // No need for the relation plugin as gutenberg media library opener
        // doesn't check the access to the root entity.
        '0' 
      );

      $state->set('media_library_opener_parameters', array_merge($state->getOpenerParameters(), ['group_parameters' => $group_state->all()]));
      $state->set('hash', $state->getHash());
    }

    $buildUi = $this->builder->buildUi(
      $state
    );
    $this->moduleHandler->alter('gutenberg_media_library_view', $buildUi);

    return $this->renderer->render($buildUi);
  }

}
